/***************************************************************************
 *   Copyright (C) 1980-2005 Artur Wiebe                                   *
 *   artur@wiebenet.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "newgamedlg.h"
#include "pdn.h"
#include "pdnpreview.h"
#include "config.h"

#include "player.h"
#include "computerplayer.h"
#include "humanplayer.h"
#include "networkplayer.h"

#include <qframe.h>
#include <qradiobutton.h>
#include <qlayout.h>
#include <qtimer.h>
#include <qhbox.h>
#include <qgrid.h>
#include <qvalidator.h>

#include <stdlib.h>


#define BEGINNER	2
#define NOVICE		4
#define AVERAGE		6
#define GOOD		7
#define EXPERT		8
#define MASTER		9

#define COMPUTER	0
#define HUMAN		1
#define SERVER		2
#define CLIENT		3

#define TIMEOUT		300

#define WHITE		1
#define BLACK		0


myNewGameDlg::myNewGameDlg(QWidget* parent)
    : QDialog(parent, 0, true)
{
    m_player = m_opponent = 0;

    setCaption(tr("New Game")+QString(" - "APPNAME));


    /*
     * options.
     */
    m_options = new QWidgetStack(this);
    m_options->addWidget(create_human_options());
    m_options->addWidget(create_computer_options());
    m_options->addWidget(create_server_options());
    m_options->addWidget(create_client_options());

    /*
     * buttons.
     */
    start_button = new QPushButton(tr("&Start"), this);
    QPushButton* cn = new QPushButton(tr("&Cancel"), this);
    connect(start_button, SIGNAL(clicked()), this, SLOT(slotStart()));
    connect(cn, SIGNAL(clicked()), this, SLOT(slotReject()));


    /*
     * top layout.
     */
    QHBoxLayout* hb = new QHBoxLayout(5);
    hb->addWidget(create_global());
    hb->addWidget(m_options);

    QHBoxLayout* bl = new QHBoxLayout(5);
    bl->addStretch();
    bl->addWidget(start_button);
    bl->addWidget(cn);

    QVBoxLayout* topl = new QVBoxLayout(this, 5, 5);
    topl->addLayout(hb);
    topl->addLayout(bl);
}


myNewGameDlg::~myNewGameDlg()
{
}


QWidget* myNewGameDlg::create_human_options()
{
    m_human.frm = new QGroupBox(tr("Human"), this);
    return m_human.frm;
}


QWidget* myNewGameDlg::create_global()
{
    QVBox* frm = new QVBox(this);
    frm->setSpacing(5);

    //
    QGroupBox* vb1 = new QGroupBox(1, Qt::Horizontal, tr("Player One"), frm);

    m_global.player1 = new QLineEdit(vb1);

    m_global.rules = new QButtonGroup(1, Qt::Horizontal, tr("Rules"), vb1);
    m_global.rules->insert(new QRadioButton(PdnPreview::typeToString(ENGLISH),
		m_global.rules), ENGLISH);
    m_global.rules->insert(new QRadioButton(PdnPreview::typeToString(RUSSIAN),
		m_global.rules), RUSSIAN);

    m_global.white = new QCheckBox(tr("White"), vb1);
    m_global.white->setChecked(true);

    //
    QGroupBox* vb2 = new QGroupBox(1, Qt::Horizontal, tr("Player Two"), frm);

    m_global.player2 = new QLineEdit(vb2);

    m_global.game = new QListBox(vb2);
    connect(m_global.game, SIGNAL(highlighted(int)),
	    this, SLOT(slotGame(int)));
    m_global.game->insertItem(tr("Computer"), COMPUTER);
    m_global.game->insertItem(tr("Human"), HUMAN);
    m_global.game->insertItem(tr("Network - New Game"), SERVER);
    m_global.game->insertItem(tr("Network - Join Game"), CLIENT);
    
/*
    // layout.
    QVBoxLayout* l = new QVBoxLayout(frm, 5);
    frm->addWidget(vb1);
    frm->addWidget(vb2);
    */

    m_global.frm = frm;
    return m_global.frm;
}


QWidget* myNewGameDlg::create_computer_options()
{
    QGroupBox* frm = new QGroupBox(1, Qt::Horizontal, tr("Computer"),
	    this);

    m_computer.skills = new QButtonGroup(tr("Skill"), frm);
    m_computer.skills->setColumnLayout(2, Qt::Vertical);
    connect(m_computer.skills, SIGNAL(clicked(int)),
	    this, SLOT(slotSkills(int)));

    m_computer.skills->insert(new QRadioButton(tr("Beginner"),
		m_computer.skills), BEGINNER);
    m_computer.skills->insert(new QRadioButton(tr("Novice"),
		m_computer.skills), NOVICE);
    m_computer.skills->insert(new QRadioButton(tr("Average"),
		m_computer.skills), AVERAGE);
    m_computer.skills->insert(new QRadioButton(tr("Good"),
		m_computer.skills), GOOD);
    m_computer.skills->insert(new QRadioButton(tr("Expert"),
		m_computer.skills), EXPERT);
    m_computer.skills->insert(new QRadioButton(tr("Master"),
		m_computer.skills), MASTER);

    m_computer.frm = frm;
    return m_computer.frm;
}


QWidget* myNewGameDlg::create_server_options()
{
    QGroupBox* frm = new QGroupBox(1, Qt::Horizontal, tr("Server"), this);

    //
    QGrid* grid = new QGrid(2, frm);
    grid->setSpacing(5);

    new QLabel(tr("Address:"), grid);
    QHBox* addrbox = new QHBox(grid);
    m_server.addr = new QLineEdit(addrbox);
    m_server.addr->setEnabled(false);
    new QLabel(":", addrbox);
    m_server.port = new QLineEdit(addrbox);
    m_server.port->setValidator(new QIntValidator(1024, 65535, this));
 
    new QLabel(tr("Servername:"), grid);
    m_server.name = new QLineEdit(grid);

/*
    new QLabel(tr("Serverinfo:"), grid);
    m_server.serverinfo = new QLineEdit(grid);
    m_server.serverinfo->setReadOnly(true);
*/

    //
    m_server.msg = new QLabel(frm);
    m_server.msg->setAlignment(Qt::AlignCenter);
    m_server.msg->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_server.msg->setMinimumHeight(100);

    m_server.accept = new QPushButton(tr("Accept"), frm);
    connect(m_server.accept, SIGNAL(clicked()), this, SLOT(slotServerAccept()));

    m_server.frm = frm;
    return m_server.frm;
}


QWidget* myNewGameDlg::create_client_options()
{
    QGroupBox* frm = new QGroupBox(1, Qt::Horizontal, tr("Client"), this);

    //
    QGrid* grid = new QGrid(2, frm);
    grid->setSpacing(5);

    new QLabel(tr("Address:"), grid);
    QHBox* addrbox = new QHBox(grid);
    m_client.addr = new QLineEdit(addrbox);
    new QLabel(":", addrbox);
    m_client.port = new QLineEdit(addrbox);
    m_client.port->setValidator(new QIntValidator(1024, 65535, this));
 
    /*
    new QLabel(tr("Servername:"), grid);
    m_client.servername = new QLineEdit(grid);
    */

    new QLabel(tr("Serverinfo:"), grid);
    m_client.serverinfo = new QLineEdit(grid);
    m_client.serverinfo->setReadOnly(true);

    //
    m_client.msg = new QLabel(frm);
    m_client.msg->setAlignment(Qt::AlignCenter);
    m_client.msg->setFrameStyle(QFrame::Panel | QFrame::Sunken);
    m_client.msg->setMinimumHeight(100);

    m_client.frm = frm;
    return m_client.frm;
}


void myNewGameDlg::slotReject()
{
    if(m_player)
	delete m_player;
    if(m_opponent)
	delete m_opponent;

    reject();
}


void myNewGameDlg::slotStart()
{
    start_button->setEnabled(false);
    m_global.frm->setEnabled(false);

    if(m_player)
	delete m_player;
    if(m_opponent)
	delete m_opponent;

    switch(m_global.game->currentItem()) {
    case COMPUTER:	start_a_computer_game();break;
    case SERVER:	start_a_server_game();	break;
    case CLIENT:	start_a_client_game();	break;
    case HUMAN:		start_a_human_game();	break;
    default:	qDebug("slotStart() err.");	break;
    }
}


void myNewGameDlg::slotSkills(int)
{
//aw??    m_global.player2->setText(APPNAME"."+QString::number(id));
    m_global.player2->setText(APPNAME"."+m_computer.skills->selected()->text());
}


void myNewGameDlg::slotGame(int id)
{
    start_button->setEnabled(true);
    m_global.frm->setEnabled(true);

    if(m_global.old_game==HUMAN) {
	m_cfg_player2 = m_global.player2->text();
    }

    m_global.old_game = id;

    switch(id) {
    case COMPUTER:
	m_global.player2->setReadOnly(true);
	slotSkills(m_computer.skills->selectedId());

	m_options->raiseWidget(m_computer.frm);
	m_global.rules->setEnabled(true);
	m_global.white->setEnabled(true);
	break;

    case SERVER:
	m_global.player2->setReadOnly(true);
	m_global.player2->clear();

	m_server.port->setEnabled(true);
	m_server.name->setEnabled(true);
	m_server.msg->clear();
	m_server.accept->setEnabled(false);

	m_global.rules->setEnabled(true);
	m_global.white->setEnabled(true);
	m_options->raiseWidget(m_server.frm);
	break;

    case CLIENT:
	m_global.player2->setReadOnly(true);
	m_global.player2->clear();

	m_client.addr->setEnabled(true);
	m_client.port->setEnabled(true);
	m_client.msg->clear();

	m_options->raiseWidget(m_client.frm);
	m_global.rules->setEnabled(false);
	m_global.white->setEnabled(false);
	break;

    case HUMAN:
	m_global.player2->setReadOnly(false);
	m_global.player2->setText(m_cfg_player2);

	m_options->raiseWidget(m_human.frm);
	m_global.rules->setEnabled(true);
	m_global.white->setEnabled(true);
	break;

    default:
	qDebug("slotGame() err.");
	break;
    }
}


void myNewGameDlg::writeSettings(QSettings* cfg)
{
    cfg->writeEntry(CFG_KEY"Skill", skill());
    cfg->writeEntry(CFG_KEY"Rules", rules());
    cfg->writeEntry(CFG_KEY"White", m_global.white->isChecked());

    cfg->writeEntry(CFG_KEY"ServerIP", m_server.addr->text());
    cfg->writeEntry(CFG_KEY"ServerPort", m_server.port->text());
    cfg->writeEntry(CFG_KEY"ServerName", m_server.name->text());

    cfg->writeEntry(CFG_KEY"ClientIP", m_client.addr->text());
    cfg->writeEntry(CFG_KEY"ClientPort", m_client.port->text());

    cfg->writeEntry(CFG_KEY"Player1", m_global.player1->text());
    cfg->writeEntry(CFG_KEY"Player2", m_cfg_player2);
}


void myNewGameDlg::readSettings(QSettings* cfg)
{
    m_computer.skills->setButton(cfg->readNumEntry(CFG_KEY"Skill", BEGINNER));
    m_global.rules->setButton(cfg->readNumEntry(CFG_KEY"Rules", ENGLISH));
    m_global.white->setChecked(cfg->readBoolEntry(CFG_KEY"White", true));

    m_server.addr->setText(cfg->readEntry(CFG_KEY"ServerIP", "localhost"));
    m_server.port->setText(cfg->readEntry(CFG_KEY"ServerPort", "54321"));
    m_server.name->setText(cfg->readEntry(CFG_KEY"ServerName",
		APPNAME" "VERSION));

    m_client.addr->setText(cfg->readEntry(CFG_KEY"ClientIP", "localhost"));
    m_client.port->setText(cfg->readEntry(CFG_KEY"ClientPort", "54321"));

    m_global.player1->setText(cfg->readEntry(CFG_KEY"Player1", getenv("USER")));
    m_cfg_player2 = cfg->readEntry(CFG_KEY"Player2", "Player2");
}


void myNewGameDlg::start_a_computer_game()
{
    // white=true
    // second_player=false
    m_player = (myPlayer*)new myHumanPlayer(m_global.white->isChecked(),
	    rules(), m_global.player1->text(), false);

    // white=false
    m_opponent = (myPlayer*)new myComputerPlayer(!m_player->isWhite(), rules(),
	    m_global.player2->text());

    m_player->setOpponent(m_opponent);
    m_opponent->setOpponent(m_player);

    //
    accept();
}


void myNewGameDlg::start_a_human_game()
{
    // white = true
    // second_player=false
    m_player = (myPlayer*)new myHumanPlayer(m_global.white->isChecked(),
	    rules(), m_global.player1->text(), false);
    // white=false
    // second_player=true
    m_opponent = (myPlayer*)new myHumanPlayer(!m_player->isWhite(), rules(),
	    m_global.player2->text(), true);

    m_player->setOpponent(m_opponent);
    m_opponent->setOpponent(m_player);

    //
    accept();
}


void myNewGameDlg::start_a_server_game()
{
    m_server.port->setEnabled(false);
    m_server.name->setEnabled(false);

    // white=true
    // second_player=false
    m_player = (myPlayer*)new myHumanPlayer(m_global.white->isChecked(),
	    rules(), m_global.player1->text(), false);
    // white=false
    // server=true
    m_opponent = (myPlayer*)new myNetworkPlayer(!m_player->isWhite(), rules(),
	    true, m_server.addr->text(), m_server.port->text().toInt());
    connect(m_opponent, SIGNAL(ready(bool)), this,SLOT(slotServerReady(bool)));
    connect(m_opponent, SIGNAL(steady(bool)),this,SLOT(slotServerSteady(bool)));
    connect(m_opponent, SIGNAL(stopped()), this, SLOT(slotServerStopped()));

    m_opponent->setSite(m_server.name->text());

    m_player->setOpponent(m_opponent);
    m_opponent->setOpponent(m_player);

    m_server.msg->setText(tr("Wait..."));
}


void myNewGameDlg::start_a_client_game()
{
    m_client.addr->setEnabled(false);
    m_client.port->setEnabled(false);

    // white=true, set by the server
    // second_player=false
    m_player = (myPlayer*)new myHumanPlayer(true, rules(),
	    m_global.player1->text(), false);
    // white=true, set by the server
    // server=false
    m_opponent = (myPlayer*)new myNetworkPlayer(true, rules(), false,
	    m_client.addr->text(), m_client.port->text().toInt());
    connect(m_opponent, SIGNAL(ready(bool)), this,SLOT(slotClientReady(bool)));
    connect(m_opponent, SIGNAL(steady(bool)),this,SLOT(slotClientSteady(bool)));
    connect(m_opponent, SIGNAL(stopped()), this, SLOT(slotClientStopped()));

    m_player->setOpponent(m_opponent);
    m_opponent->setOpponent(m_player);

    m_client.msg->setText(tr("Wait..."));
}


void myNewGameDlg::slotServerReady(bool ready)
{
    if(ready) {
	m_server.msg->setText(tr("Waiting for client to connect..."));
    } else
	m_server.msg->setText(tr("Could not create a game on the server."));
}


void myNewGameDlg::slotServerSteady(bool)
{
    m_global.player2->setText(m_opponent->name());
//aw??    m_network.serverinfo->setText(m_opponent->name());
    m_server.msg->setText(tr("Client connected."));
    //
    m_server.accept->setEnabled(true);
}


void myNewGameDlg::slotServerStopped()
{
    m_server.accept->setEnabled(false);
    m_server.msg->setText(tr("Client disconnected."));
}


void myNewGameDlg::slotClientReady(bool ready)
{
    if(ready) {
	m_client.msg->setText(tr("Waiting for server to accept..."));
	m_global.player2->setText(m_opponent->name());
	m_client.serverinfo->setText(QString("%1, %2")
		.arg(m_opponent->site())
		.arg(PdnPreview::typeToString(m_opponent->rules())));
    } else
	m_client.msg->setText(tr("Could not join the game."));
}


void myNewGameDlg::slotClientSteady(bool accepted)
{
    if(accepted) {
	//
	//m_net_msg->setEnabled(true);
	accept();
    } else
	m_client.msg->setText(tr("Server denied."));
}


void myNewGameDlg::slotClientStopped()
{
    m_client.msg->setText(tr("Connection aborted."));
}


int myNewGameDlg::get(bool with_dialog)
{
    if(with_dialog) {
	m_player = m_opponent = 0;
	slotGame(m_global.game->currentItem());
	return exec();
    }

    m_global.game->setCurrentItem(COMPUTER);
    start_a_computer_game();
    return -1;
}


void myNewGameDlg::slotServerAccept()
{
    m_opponent->acceptPlay(true);
    accept();
}

/*
void myNewGameDlg::slotServerDeny()
{
}
*/

