/***************************************************************************
 *   Copyright (C) 1980-2005 Artur Wiebe                                   *
 *   artur@wiebenet.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _NEWGAMEDLG_H_
#define _NEWGAMEDLG_H_

#include <qdialog.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qsettings.h>
#include <qbuttongroup.h>
#include <qpushbutton.h>
#include <qlistbox.h>
#include <qwidgetstack.h>
#include <qcheckbox.h>
#include <qvbox.h>


class myPlayer;


class myNewGameDlg : public QDialog
{
    Q_OBJECT

public:
    myNewGameDlg(QWidget* parent);
    ~myNewGameDlg();

    // important, call this instead of exec().
    // with_dialog=true for a dialog
    // with_dialog=false for a standard game against computer.
    int get(bool with_dialog);

    // playing against the computer
    int skill() const { return m_computer.skills->selectedId(); }

    // creating/joining a network game
    /*
    QString addr() const { return m_network.addr->text(); }
    int port() { return m_network.port->text().toInt(); }
    */

    // for settings
    void writeSettings(QSettings*);
    void readSettings(QSettings*);

    myPlayer* first() const { return m_player; }
    myPlayer* second() const { return m_opponent; }


private:
    int rules() const { return m_global.rules->selectedId(); }

private slots:
    void slotGame(int id);
    void slotSkills(int id);

    void slotReject();
    void slotStart();

    void slotServerReady(bool);
    void slotServerSteady(bool);
    void slotServerStopped();

    void slotClientReady(bool);
    void slotClientSteady(bool);
    void slotClientStopped();

    void slotServerAccept();

private:
    QWidget* create_global();
    QWidget* create_human_options();
    QWidget* create_computer_options();
    QWidget* create_server_options();
    QWidget* create_client_options();

    void start_a_computer_game();
    void start_a_server_game();
    void start_a_client_game();
    void start_a_human_game();

private:
    // global stuff
    struct global_struct {
	QVBox* frm;
	QLineEdit* player1;
	QLineEdit* player2;
	QListBox* game;
	QButtonGroup* rules;
	QCheckBox* white;
	int old_game;
    };
    struct global_struct m_global;

    // human stuff
    struct human_struct {
	QGroupBox* frm;
    };
    struct human_struct m_human;

    // computer stuff
    struct computer_stuct {
	QGroupBox* frm;
	QButtonGroup* skills;
    };
    struct computer_stuct m_computer;

    // server stuff
    struct server_struct {
	QGroupBox* frm;
	QLineEdit* addr;
	QLineEdit* port;
	QLineEdit* name;
//	QLineEdit* serverinfo;
	QLabel* msg;
	QPushButton* accept;
    };
    struct server_struct m_server;

    // client stuff
    struct client_struct {
	QGroupBox* frm;
	QLineEdit* addr;
	QLineEdit* port;
	QLineEdit* serverinfo;
	QLabel* msg;
//	QPushButton* accept;
    };
    struct client_struct m_client;

    QWidgetStack* m_options;

    QPushButton* start_button;

    myPlayer* m_player;
    myPlayer* m_opponent;

    QString m_cfg_player2;
};

#endif

