/***************************************************************************
 *   Copyright (C) 1980-2005 by Artur Wiebe                                *
 *   artur@wiebenet.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef _PLAYER_H_
#define _PLAYER_H_

#include "checkers.h"
#include "field.h"

#include <qobject.h>


class myPlayer : public QObject
{
    Q_OBJECT

public:
    myPlayer(bool white, int rules)
	: m_opponent(0), m_white(white), m_rules(rules) {}

    // get information
    bool isWhite() const { return m_white; }
    void setWhite(bool b) { m_white = b; }
    int rules() const { return m_rules; }
    const QString& name() const { return m_name; }
    virtual bool isHuman() const { return false; }
    virtual bool isNetwork() const { return false; }
    virtual bool isServer() const { return false; }
    const QString& site() const { return m_site; }
    void setSite(const QString& s) { m_site = s; }

    // for server/client communication
    virtual void sendNextRound() {}
    virtual void acceptPlay(bool) {}

    virtual void yourTurn(Checkers* game) = 0;
    // return false on incorrect course.
    // set select to true to select the select, or unselect.
    // QString contains error msg.
    virtual bool fieldClicked(int, bool*,QString&) { return true; }
    virtual void sendMove(const QString&) {}
    // computerplayer terminates his thinking thread.
    // humanplayer
    // networkplayer disconnects from the server.
    virtual void stop() = 0;

    virtual void undo() {}
    virtual bool isUndoAvailable() const { return false; }

    void setOpponent(myPlayer* o) { m_opponent=o; }
    myPlayer* opponent() const { return m_opponent; }

public slots:
    virtual void sendMessage(const QString &) {}

signals:
    // emitted true when created/joined game.
    // emitted false when timed out.
    void ready(bool);

    // serverside: emitted when the client sends a play wish.
    // 			paramater no meaning.
    // clientside: emitted true for an accepted play.
    // 		   emitted false a denied play.
    void steady(bool);

    // emmited when the other side is closed.
    void stopped();

//    void readyToPlay();

    void message(const QString& text);
    // type=ENGLISH,RUSSIAN,..
    // is_white=player is white or not.
//aw???    void changeGameType(int type, bool is_white);
    //
    void nextRound();

    // emitted when move is done, provides the complete board converted to
    // string.
    void moveDone(const QString&);

protected:
    void setName(const QString& n) { m_name = n; }
    void setRules(int r) { m_rules = r; }

private:
    QString m_name;
    myPlayer* m_opponent;
    bool m_white;
    int m_rules;

    QString m_site;	// pdn conform
};


#endif
