/***************************************************************************
 *   Copyright (C) 2002-2003 Andi Peredri                                  *
 *   andi@ukr.net                                                          *
 *   Copyright (C) 1980-2005 Artur Wiebe                                   *
 *   artur@wiebenet.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "pdn.h"
#include "pdnpreview.h"
#include "toplevel.h"
#include "newgamedlg.h"
#include "console.h"
#include "view.h"
#include "config.h"

#include <qlabel.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qprocess.h>
#include <qsettings.h>
#include <qstatusbar.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>
#include <qapplication.h>
#include <qkeysequence.h>
#include <qaction.h>
#include <qpopupmenu.h>
#include <qtextedit.h>


myTopLevel::myTopLevel()
//    : QMainWindow(0, 0, WStyle_DialogBorder)
{
    setCaption(APPNAME);
    setIcon(QPixmap::fromMimeSource("biglogo.png"));
    setFixedSize(minimumSizeHint());

    setDockWindowsMovable(false);
    setDockMenuEnabled(false);

    m_newgame = new myNewGameDlg(this);
    make_checkers_board();

    // basic inits
    make_actions();
    restore_settings();

    undo = false;

    // new computer game.
    m_newgame->get(false);
    view->newGame(m_newgame->first(), m_newgame->second(),
	    m_newgame->skill());

    statusBar()->setSizeGripEnabled(false);
}


void myTopLevel::make_actions()
{
    // game menu actions
    gameNew = new QAction(QPixmap::fromMimeSource("logo.png"),
	    tr("&New..."), QKeySequence(tr("CTRL+N", "File|New")), this);
    connect(gameNew, SIGNAL(activated()), this, SLOT(slotNewGame()));

    gameNextRound = new QAction(QPixmap::fromMimeSource("next.png"),
	    tr("&Next Round"), QKeySequence(), this);
    connect(gameNextRound, SIGNAL(activated()), this, SLOT(slotNextRound()));

    gameStop = new QAction(QPixmap::fromMimeSource("stop.png"),
	    tr("&Stop"), QKeySequence(), this);
    connect(gameStop, SIGNAL(activated()), view, SLOT(slotStopGame()));

    gameUndo = new QAction(QPixmap::fromMimeSource("undo.png"),
	    tr("&Undo Move"), QKeySequence(tr("CTRL+Z", "File|Undo")), this);
    connect(gameUndo, SIGNAL(activated()), view, SLOT(undo()));

    QAction* gameInfo = new QAction(QPixmap::fromMimeSource("info.png"),
	    tr("&Information"), QKeySequence(), this);
    connect(gameInfo, SIGNAL(activated()), this, SLOT(slotGameInfo()));

    gameOpen = new QAction(QPixmap::fromMimeSource("fileopen.png"),
	    tr("&Open..."), QKeySequence(tr("CTRL+O", "File|Open")), this);
    connect(gameOpen, SIGNAL(activated()), this, SLOT(slotOpenGame()));

    gameSave = new QAction(QPixmap::fromMimeSource("filesave.png"),
	    tr("&Save..."), QKeySequence(tr("CTRL+S", "File|Save")), this);
    connect(gameSave, SIGNAL(activated()), this, SLOT(slotSaveGame()));

    QAction* gameQuit = new QAction(QPixmap::fromMimeSource("exit.png"),
	    tr("&Quit"), QKeySequence(tr("CTRL+Q", "File|Quit")), this);
    connect(gameQuit, SIGNAL(activated()), qApp, SLOT(closeAllWindows()));

    // view menu actions
    viewNotation = new QAction(tr("&Show Notation"), QKeySequence(), this);
    viewNotation->setToggleAction(true);
    connect(viewNotation, SIGNAL(toggled(bool)),
	    view, SLOT(slotShowNotation(bool)));

    QActionGroup* pattern_grp = new QActionGroup(this);
    pattern_grp->setExclusive(true);
    connect(pattern_grp, SIGNAL(selected(QAction*)), this, SLOT(setPattern()));
    //
    viewGreen = new QAction(tr("&Green Board"), QKeySequence(), pattern_grp);
    viewGreen->setToggleAction(true);
    //
    viewMarble = new QAction(tr("&Marble Board"), QKeySequence(), pattern_grp);
    viewMarble->setToggleAction(true);
    //
    viewWooden = new QAction(tr("&Wooden Board"), QKeySequence(), pattern_grp);
    viewWooden->setToggleAction(true);

    QActionGroup* size_grp = new QActionGroup(this);
    size_grp->setExclusive(true);
    connect(size_grp, SIGNAL(selected(QAction*)), this, SLOT(setPattern()));
    //
    viewSmall = new QAction(tr("&Small board"), QKeySequence(), size_grp);
    viewSmall->setToggleAction(true);
    //
    viewBig = new QAction(tr("&Big board"), QKeySequence(), size_grp);
    viewBig->setToggleAction(true);
    //
    viewConsole = new QAction(QPixmap::fromMimeSource("console.png"),
	    tr("&Console"), QKeySequence(), this);
    viewConsole->setToggleAction(true);
    connect(viewConsole, SIGNAL(activated()), this, SLOT(slotShowConsole()));

    settingsConsole = new QAction(tr("&Open console on new messages"),
	    QKeySequence(), this);
    settingsConsole->setToggleAction(true);
    //
    settingsKeep = new QAction(tr("&Confirm aborting current game"),
	    QKeySequence(), this);
    settingsKeep->setToggleAction(true);

    // help menu actions
    QAction* helpWhatsThis = new QAction(QPixmap::fromMimeSource("context.png"),
	    tr("What's This"), QKeySequence(tr("SHIFT+F1", "Help|WhatsThis")),
	    this);
    connect(helpWhatsThis, SIGNAL(activated()), this, SLOT(whatsThis()));

    QAction* helpRules = new QAction(tr("&Rules of Play"),
	    QKeySequence(tr("F1", "Help|Help")), this);
    connect(helpRules, SIGNAL(activated()), this, SLOT(slotHelp()));

    QAction* helpAbout = new QAction(QPixmap::fromMimeSource("logo.png"),
	    tr("&About")+QString(" "APPNAME), QKeySequence(), this);
    connect(helpAbout, SIGNAL(activated()), this, SLOT(slotAbout()));

    QAction* helpAboutQt = new QAction(tr("About &Qt"), QKeySequence(), this);
    connect(helpAboutQt, SIGNAL(activated()), qApp, SLOT(aboutQt()));

    // menus
    QPopupMenu* gameMenu = new QPopupMenu(this);
    gameNew->addTo(gameMenu);
    gameOpen->addTo(gameMenu);
    gameSave->addTo(gameMenu);
    gameMenu->insertSeparator();
    gameNextRound->addTo(gameMenu);
    gameStop->addTo(gameMenu);
    gameUndo->addTo(gameMenu);
    gameInfo->addTo(gameMenu);
    gameMenu->insertSeparator();
    gameQuit->addTo(gameMenu);

    viewMenu = new QPopupMenu(this);
    viewNotation->addTo(viewMenu);
    viewMenu->insertSeparator();
    pattern_grp->addTo(viewMenu);
    viewMenu->insertSeparator();
    size_grp->addTo(viewMenu);
    viewMenu->insertSeparator();
    viewConsole->addTo(viewMenu);

    QPopupMenu* settingsMenu = new QPopupMenu(this);
    settingsConsole->addTo(settingsMenu);
    settingsKeep->addTo(settingsMenu);

    QPopupMenu* helpMenu = new QPopupMenu(this);
    helpWhatsThis->addTo(helpMenu);
    helpRules->addTo(helpMenu);
    helpMenu->insertSeparator();
    helpAbout->addTo(helpMenu);
    helpAboutQt->addTo(helpMenu);

    // menubar
    menuBar()->insertItem(tr("&Game"), gameMenu);
    menuBar()->insertItem(tr("&View"), viewMenu);
    menuBar()->insertItem(tr("&Settings"), settingsMenu);
    menuBar()->insertSeparator();
    menuBar()->insertItem(tr("&Help"), helpMenu);

    // toolbar
    QToolBar* tb = new QToolBar(this);
    gameNew->addTo(tb);
    gameOpen->addTo(tb);
    gameSave->addTo(tb);
    tb->addSeparator();
    gameNextRound->addTo(tb);
    gameStop->addTo(tb);
    gameUndo->addTo(tb);
    gameInfo->addTo(tb);
    tb->addSeparator();
    viewConsole->addTo(tb);
}


void myTopLevel::restore_settings()
{
    QSettings cfg;
    QString str;
    QString b;

    str=cfg.readEntry(CFG_KEY"Theme", actionToPattern());
    patternToAction(str)->setOn(true);

    str=cfg.readEntry(CFG_KEY"TileSize", actionToSize());
    sizeToAction(str)->setOn(true);

    filename = cfg.readEntry(CFG_KEY"Filename");

    b = cfg.readBoolEntry(CFG_KEY"Notation", false);
    viewNotation->setOn(b);
    view->slotShowNotation(b);

    settingsConsole->setOn(cfg.readBoolEntry(CFG_KEY"AutoShowConsole", true));
    settingsKeep->setOn(cfg.readBoolEntry(CFG_KEY"ShowKeepDialog", true));

    // new game
    m_newgame->readSettings(&cfg);
}


void myTopLevel::make_checkers_board()
{
    QVBox* vbox = new QVBox(this);
    vbox->setSpacing(5);

    // widgets
    view = new myView(vbox);
    m_console = new myConsole(vbox);
    m_console->hide();

    connect(view, SIGNAL(enableUndo(bool)),
	    this, SLOT(slotEnableUndo(bool)));
    connect(view, SIGNAL(message(int, const QString&)),
	    m_console, SLOT(message(int, const QString&)));
    connect(view, SIGNAL(working(bool)),
	    this, SLOT(slotWorking(bool)));

    connect(m_console, SIGNAL(sendText(const QString&)),
	    view, SIGNAL(sendText(const QString&)));

    connect(m_console, SIGNAL(updated()), this, SLOT(slotConsoleUpdated()));

//aw???    QWhatsThis::add(frame,tr("A checkers board"));
    setCentralWidget(vbox);
}


void myTopLevel::warning(const QString& text)
{
    QMessageBox::warning(this, tr("Error")+QString(" - "APPNAME), text);
}


void myTopLevel::information(const QString& caption, const QString& text)
{
    QDialog* dlg = new QDialog(this, 0, true);
    dlg->setCaption(caption+QString(" - "APPNAME));

    // logo left
    QLabel* logo = new QLabel(dlg);
    logo->setPixmap(QPixmap::fromMimeSource("dialog.png"));

    // text editor
    QTextEdit* te = new QTextEdit(text, QString::null, dlg);
    te->setTextFormat(Qt::RichText);
    te->setReadOnly(true);
    /*
    te->setSizePolicy(QSizePolicy(QSizePolicy::Preferred,
		QSizePolicy::Preferred, true));
		*/
    te->setMinimumWidth(340);
    te->setMinimumHeight(240);

    // close button
    QPushButton* button = new QPushButton(tr("&Close"), dlg);
    connect(button, SIGNAL(clicked()), dlg, SLOT(accept()));

    // Layout.
    QHBoxLayout* hb = new QHBoxLayout(5);
    hb->addWidget(logo, 0, Qt::AlignTop);
    hb->addWidget(te, 1);

    QHBoxLayout* hb_button = new QHBoxLayout(5);
    hb_button->addStretch();
    hb_button->addWidget(button);

    QVBoxLayout* vb = new QVBoxLayout(dlg, 5);
    vb->addLayout(hb, 1);
    vb->addSpacing(5);
    vb->addLayout(hb_button);

    // go
    dlg->exec();
    delete dlg;
}


void myTopLevel::slotSaveGame()
{
    QFileDialog* dlg = new QFileDialog(this);
    PdnPreview* preview = new PdnPreview(dlg);

    dlg->setContentsPreviewEnabled(true);
    dlg->setFilter("PDN Files (*."EXT")");
    dlg->setSelection(filename);
    dlg->setCaption(tr("Save Game")+" "APPNAME);
    dlg->setMode(QFileDialog::AnyFile);

    dlg->setContentsPreview(preview, preview);
    dlg->setPreviewMode(QFileDialog::Contents);

    connect(preview, SIGNAL(selected(int)), dlg, SLOT(accept()));

    if(dlg->exec() == QDialog::Accepted) {
	filename = dlg->selectedFile();
	if(filename.right(3)!=EXT)
	    filename += "."EXT;

	Pdn pdn;
	view->getPdn(&pdn);

	if(!pdn.save(filename, preview->currentItem()))
	    warning(tr("Could not save: ")+filename);
    }

    delete dlg;
}


void myTopLevel::slotOpenGame()
{
    if(keepGame())
	return;

    QFileDialog* dlg = new QFileDialog(this);
    PdnPreview* preview = new PdnPreview(dlg);

    dlg->setContentsPreviewEnabled(true);
    dlg->setFilter("PDN Files (*."EXT")");
    dlg->setSelection(filename);
    dlg->setCaption(tr("Open Game")+" - "APPNAME);

    dlg->setContentsPreview(preview, preview);
    dlg->setPreviewMode(QFileDialog::Contents);

    connect(preview, SIGNAL(selected(int)), dlg, SLOT(accept()));

    // go
    if(dlg->exec() == QDialog::Accepted) {
	// open
	filename = dlg->selectedFile();

	Pdn pdn;
	pdn.open(filename);

	int item = preview->currentItem();
	if(item == -1)
	    item=0;

	if(pdn.parse(item))
    	    view->openGame(&pdn, preview->boardOnly());
	else
	    warning(tr("Could not load: ")+filename);
    }

    delete dlg;
}


void myTopLevel::closeEvent(QCloseEvent* e)
{
    if(keepGame()) {
	e->ignore();
	return;
    }

    QSettings config;
    QString entry;

    config.writeEntry(CFG_KEY"Theme", actionToPattern());

    config.writeEntry(CFG_KEY"TileSize", actionToSize());

    config.writeEntry(CFG_KEY"Filename", filename);
    config.writeEntry(CFG_KEY"Notation", viewNotation->isOn());

    config.writeEntry(CFG_KEY"AutoShowConsole", settingsConsole->isOn());
    config.writeEntry(CFG_KEY"ShowKeepDialog", settingsKeep->isOn());

    // new game
    m_newgame->writeSettings(&config);

    QMainWindow::closeEvent(e);
}


void myTopLevel::setPattern()
{
    view->setPattern(actionToPattern(), actionToSize());
}


void myTopLevel::slotShowConsole()
{
    if(viewConsole->isOn())
	m_console->show();
    else
	m_console->hide();
}


void myTopLevel::slotGameInfo()
{
    Pdn pdn;
    view->getPdn(&pdn);

    QString text = "<pre>"
	  "Event:  " + pdn.get(Pdn::Event)	+ "<br>"
    	+ "Site:   " + pdn.get(Pdn::Site)	+ "<br>"
	+ "Date:   " + pdn.get(Pdn::Date)	+ "<br>"
	+ "Round:  " + pdn.get(Pdn::Round)	+ "<br>"
	+ "White:  " + pdn.get(Pdn::White)	+ "<br>"
	+ "Black:  " + pdn.get(Pdn::Black)	+ "<br>"
	+ "Result: " + pdn.get(Pdn::Result)	+ "<br>"
       	+ "Type:   " + pdn.get(Pdn::Type)	+ "</pre>";

    information(tr("Game Info"), text);
}


void myTopLevel::slotHelp()
{
    QString text = 
	tr("<p>In the beginning of game you have 12 checkers (men). "
	"White always moves first. "
	"The men move forward only. The men can capture:"
	"<ul>"
	"<li>by jumping forward only (english rules);"
	"<li>by jumping forward or backward (russian rules)."
	"</ul>"
	"<p>A man which reaches the far side of the board becomes a king. "
	"The kings move forward or backward:"
	"<ul>"
	"<li>to one square only (english rules);"
	"<li>to any number of squares (russian rules)."
	"</ul>"
	"<p>The kings capture by jumping forward or backward. "
	"Whenever a player is able to make a capture he must do so.");

    information(tr("Rules of Play"), text);
}


void myTopLevel::slotAbout()
{
    QString text =
	APPNAME", a board game.<br>Version "VERSION"<br><br>"
	COPYRIGHT"<br>"
      	"<a href=\""HOMEPAGE"\">"HOMEPAGE"</a><br><br>"
	"Contributors:<br>"CONTRIBS"<br><br>"
	"This program is distributed under the terms "
	"of the GNU General Public License.";

    information(tr("About"), text);
}


void myTopLevel::slotNewGame()
{
    if(keepGame())
	return;

    if(m_newgame->get(true)==QDialog::Accepted) {
	view->newGame(m_newgame->first(), m_newgame->second(),
		m_newgame->skill());
    }
}


void myTopLevel::slotEnableUndo(bool b)
{
    undo = b;
    gameUndo->setEnabled(b);
}


void myTopLevel::slotWorking(bool working)
{
    bool disable = !working;

    gameNew->setEnabled(disable);
    gameUndo->setEnabled(disable);
    gameSave->setEnabled(disable);

    if(view->isNetworkGame()) {
	gameOpen->setEnabled(false);
	gameNextRound->setEnabled(view->isServer() && !view->isAborted());
	slotEnableUndo(false);
	gameStop->setEnabled(!view->isAborted());

    } else {
	gameNextRound->setEnabled(!view->isAborted());
	gameOpen->setEnabled(disable);
	gameStop->setEnabled(!disable);

	if(working) {
	    bool old_undo = undo;
	    slotEnableUndo(false);
	    undo = old_undo;
	} else
	    slotEnableUndo(undo);
    }

    if(disable) {
	view->setEnabled(true);
	view->setCursor(Qt::ArrowCursor);
    } else {
	view->setEnabled(false);
	view->setCursor(Qt::WaitCursor);
    }
}


bool myTopLevel::keepGame()
{
    if(!settingsKeep->isOn() || view->isAborted())
	return false;

    int answer = QMessageBox::question(this, tr("Abort Game?")
	    +QString(" - "APPNAME),
	    tr("Current game will be lost if you continue.\n"
		"Do you really want to discard it?"),
	    QMessageBox::Yes, QMessageBox::No);

    if(answer == QMessageBox::Yes)
	return false;

    return true;
}


void myTopLevel::slotNextRound()
{
    if(!keepGame())
	view->slotNextRound();
}


QString myTopLevel::actionToPattern()
{
    if(viewGreen->isOn())
	return "green";
    else if(viewMarble->isOn())
	return "marble";

    return "wooden";
}


QAction* myTopLevel::patternToAction(const QString& pat)
{
    if(pat=="green")
	return viewGreen;
    else if(pat=="marble")
	return viewMarble;

    return viewWooden;
}


QString myTopLevel::actionToSize()
{
    if(viewBig->isOn())
	return "big";
    return "small";
}


QAction* myTopLevel::sizeToAction(const QString& size)
{
    if(size=="big")
	return viewBig;
    return viewSmall;
}


void myTopLevel::slotConsoleUpdated()
{
    if(settingsConsole->isOn() && !viewConsole->isOn()) {
	viewConsole->setOn(true);
	slotShowConsole();
    }
}
